/*
 * Copyright (C)2010-2012 Henning Karlsen. All right reserved.
 * Copyright (C)2012 Todor Todorov.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to:
 *
 * Free Software Foundation, Inc.
 * 51 Franklin St, 5th Floor, Boston, MA 02110-1301, USA
 *
 *********************************************************************/
#include "hx8340bs.h"
#include "helpers.h"

HX8340S_LCD::HX8340S_LCD( PinName CS, PinName RESET, PinName SCL, PinName SDI, PinName BL )
    : LCD( 176, 220, CS, NC, RESET ), _lcd_pin_scl( SCL ), _lcd_pin_sdi( SDI )
{
    if ( BL != NC ) _lcd_pin_bl = new DigitalOut( BL );
    else _lcd_pin_bl = 0;
}
 
void HX8340S_LCD::Initialize( orientation_t orientation )
{
    _orientation = orientation;
    
    wait_ms( 100 );
    _lcd_pin_reset = HIGH;
    wait_ms( 5 );
    _lcd_pin_reset = LOW;
    wait_ms( 15 );
    _lcd_pin_reset = HIGH;
    _lcd_pin_cs = HIGH;
    _lcd_pin_scl = HIGH;
    _lcd_pin_sdi = HIGH;
    if ( _lcd_pin_bl != 0 )
        *_lcd_pin_bl = HIGH;
    wait_ms( 55 );
    
    Activate();
    WriteCmd( 0xC1 );
    WriteByteData( 0xFF );
    WriteByteData( 0x83 );
    WriteByteData( 0x40 );
    WriteCmd( 0x11 );
    wait_ms( 160 );
    WriteCmd( 0xCA );
    WriteByteData( 0x70 );
    WriteByteData( 0x00 );
    WriteByteData( 0xD9 );
    WriteCmd( 0xB0 );
    WriteByteData( 0x01 );
    WriteByteData( 0x11 );

    WriteCmd( 0xC9 );
    WriteByteData( 0x90 );
    WriteByteData( 0x49 );
    WriteByteData( 0x10 );
    WriteByteData( 0x28 );
    WriteByteData( 0x28 );
    WriteByteData( 0x10 );
    WriteByteData( 0x00 );
    WriteByteData( 0x06 );
    wait_ms( 20 );
    WriteCmd( 0xC2 );
    WriteByteData( 0x60 );
    WriteByteData( 0x71 );
    WriteByteData( 0x01 );
    WriteByteData( 0x0E );
    WriteByteData( 0x05 );
    WriteByteData( 0x02 );
    WriteByteData( 0x09 );
    WriteByteData( 0x31 );
    WriteByteData( 0x0A );

    WriteCmd( 0xc3 );
    WriteByteData( 0x67 );
    WriteByteData( 0x30 );
    WriteByteData( 0x61 );
    WriteByteData( 0x17 );
    WriteByteData( 0x48 );
    WriteByteData( 0x07 );
    WriteByteData( 0x05 );
    WriteByteData( 0x33 );
    wait_ms( 10 );
    WriteCmd( 0xB5 );
    WriteByteData( 0x35 );
    WriteByteData( 0x20 );
    WriteByteData( 0x45 );

    WriteCmd( 0xB4 );
    WriteByteData( 0x33 );
    WriteByteData( 0x25 );
    WriteByteData( 0x4c );
    wait_ms( 10 );
    WriteCmd( 0x3a );
    WriteByteData( 0x05 );
    WriteCmd( 0x29 );
    wait_ms( 10 );
    WriteCmd( 0x2a );
    WriteByteData( 0x00 );
    WriteByteData( 0x00 );
    WriteByteData( 0x00 );
    WriteByteData( 0xaf );
    WriteCmd( 0x2b );
    WriteByteData( 0x00 );
    WriteByteData( 0x00 );
    WriteByteData( 0x00 );
    WriteByteData( 0xdb );
    WriteCmd( 0x2c );
    Deactivate();
}

void HX8340S_LCD::Sleep( void )
{
    Activate();
    WriteCmd( 0x28 );
    wait_ms( 10 );
    WriteCmd( 0x10 );
    wait_ms( 125 );
    if ( _lcd_pin_bl != 0 )
        *_lcd_pin_bl = LOW;
    Deactivate();
}

void HX8340S_LCD::WakeUp( void )
{
    Activate();
    WriteCmd( 0x29 );
    wait_ms( 10 );
    WriteCmd( 0x11 );
    wait_ms( 125 );
    if ( _lcd_pin_bl != 0 )
        *_lcd_pin_bl = HIGH;
    Deactivate();
}

void HX8340S_LCD::WriteCmd( unsigned short cmd )
{
    _lcd_pin_sdi = LOW;
    pulseLow( _lcd_pin_scl );
    serializeByte( cmd & 0xFF );
}

void HX8340S_LCD::WriteData( unsigned short data )
{
    _lcd_pin_sdi = HIGH;
    pulseLow( _lcd_pin_scl );
    serializeByte( ( data >> 8 ) & 0xFF );
    _lcd_pin_sdi = HIGH;
    pulseLow( _lcd_pin_scl );
    serializeByte( data & 0xFF );
}

void HX8340S_LCD::WriteByteData( unsigned char data )
{
    _lcd_pin_sdi = HIGH;
    pulseLow( _lcd_pin_scl );
    serializeByte( data );
}

void HX8340S_LCD::SetXY( uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2 )
{
    if ( _orientation == LANDSCAPE )
    {
        swap( uint16_t, x1, y1 )
        swap( uint16_t, x2, y2 )
        y1 = _disp_height - y1;
        y2 = _disp_height - y2;
        swap( uint16_t, y1, y2 )
    }

    WriteCmdData( 0x2a, x1 );
    WriteData( x2 );
    WriteCmdData( 0x2b, y1 );
    WriteData( y2 );
    WriteCmd( 0x2c );
}

void HX8340S_LCD::serializeByte( unsigned char data )
{
    for ( int i = 0; i < 8; i++ )
    {
        if ( data & 0x80 ) _lcd_pin_sdi = HIGH;
        else _lcd_pin_sdi = LOW;
        pulseLow( _lcd_pin_scl );
        data = data << 1;
    }
}
